/*
 * SortAnimation.java
 *
 * This class animates a running sorting algorithm
 */

import java.awt.*;
import java.util.Random;

public class SortAnimation extends java.applet.Applet implements Runnable {
	// this many elements in the array to be sorted

	final int n = 300;

	// a thread to handle the animation

	Thread thread;

	// the name of the sorting method, e.g. bubble, selection, etc.

	String method;

	// the array to be sorted

	Integer 
		v[], 

	// a copy of the array used for erasing the previous frame

		oldv[];

	// constructor

	SortAnimation (String _method) {
		method = _method;
	}

	// initialization

	public void init() {

		// allocate new array of integers to be sorted

		v = new Integer[n];

		// allocate copy of array for erasing

		oldv = new Integer[n];

		// initialize array elements to 0..n-1

		for (int i=0; i<v.length; i++) v[i] = i;

		// randomly shuffle array elements so we have something interesting to sort

		Random r = new Random ();
		for (int i=0; i<v.length; i++) {

			// get a random index between 0..n-1

			Integer j = Math.abs (r.nextInt() % v.length);

			// swap this random element with the i'th element

			Integer t = v[i];
			v[i] = v[j];
			v[j] = t;
		}

		// copy the array

		for (int i=0; i<v.length; i++) oldv[i] = v[i];
	}

	// start the applet running

	public void start () {

		// make a new thread for animation

		thread = new Thread(this);

		// start the thread

		thread.start();
	}

	// draw the array; called from the sorting algorithms themselves

	public void drawArray () {

		// repaint the screen

		repaint ();

		// sleep for 50 milliseconds

		try {
			Thread.sleep(50);
		} catch (InterruptedException e) {
			return;
		}
	
		// copy the array just drawn so we can erase it later

		for (int i=0; i<v.length; i++) oldv[i] = v[i];
	}

	// run the applet

	public void run () {

		// if we are running the animation thread...

		if (Thread.currentThread() == thread) {

			// get a sorting algorithm

			AnimatedSort s;
			if (method.equals ("bubble"))
				s = new AnimatedBubbleSort ();
			else if (method.equals ("selection"))
				s = new AnimatedSelectionSort ();
			else if (method.equals ("insertion"))
				s = new AnimatedInsertionSort ();
			else if (method.equals ("quicksort"))
				s = new AnimatedQuickSort ();
			else {
				System.err.println ("unknown sorting method " + method + ", assuming bubble");
				s = new AnimatedBubbleSort ();
			}

			// run the sorting algorithm; it will periodically call drawArray()

			s.sort (v, this);
		}
	}

	// draw or erase the array

	private void drawv (Graphics g, Integer v[], boolean erase) {

		// find the size of the window

		Rectangle r = g.getClipBounds();

		// figure out how much to scale pixels

		double xscale = r.width / (double) n;
		double yscale = r.height / (double) n;
		if (xscale < 1) xscale = 1;
		if (yscale < 1) yscale = 1;

		// draw each element of the array

		for (int i=0; i<v.length; i++) {

			// if we want to erase the pixel, use the background color,
			// otherwise draw in black

			if (erase)
				g.setColor (getBackground());
			else {
				g.setColor (Color.black);
				oldv[i] = v[i];
			}

			// make a scaled "pixel" of the right size

			g.fillRect ((int) (i*xscale), (int) (v[i]*yscale), (int) xscale, (int) yscale);
		}
	}

	// update the window

	public void update (Graphics g) {
	
		// erase the old array

		drawv (g, oldv, true);

		// draw the updated array

		drawv (g, v, false);

		// call the paint method (that's what update is supposed to do)

		paint (g);
	}

	public void paint (Graphics g) {  
		// nothing; it all happens in update
	} 

	// main program

	public static void main (String args[]) {

		// make sure there is an argument

		if (args.length != 1) {
			System.err.println ("Usage: java SortAnimation [ bubble | selection | insertion | quicksort ]");
			return;
		}
		// make a SortAnimation object with the user's choice of algorithm

		SortAnimation a = new SortAnimation (args[0]);

		// open up a window, stick the SortAnimation object into it

		Frame frame = new Frame ("Sort Animation");
		frame.add ("Center", a);

		// show it on the screen

		frame.show ();

		// start it running

		a.init();
		a.start();

		// make it 800 x 800

		frame.setSize (800, 800);
	}
}
